<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace RectorPrefix20210927\Symfony\Component\HttpFoundation\Session\Storage;

use RectorPrefix20210927\Symfony\Component\HttpFoundation\Session\Storage\Proxy\AbstractProxy;
/**
 * Allows session to be started by PHP and managed by Symfony.
 *
 * @author Drak <drak@zikula.org>
 */
class PhpBridgeSessionStorage extends \RectorPrefix20210927\Symfony\Component\HttpFoundation\Session\Storage\NativeSessionStorage
{
    /**
     * @param AbstractProxy|\SessionHandlerInterface|null $handler
     */
    public function __construct($handler = null, \RectorPrefix20210927\Symfony\Component\HttpFoundation\Session\Storage\MetadataBag $metaBag = null)
    {
        if (!\extension_loaded('session')) {
            throw new \LogicException('PHP extension "session" is required.');
        }
        $this->setMetadataBag($metaBag);
        $this->setSaveHandler($handler);
    }
    /**
     * {@inheritdoc}
     */
    public function start()
    {
        if ($this->started) {
            return \true;
        }
        $this->loadSession();
        return \true;
    }
    /**
     * {@inheritdoc}
     */
    public function clear()
    {
        // clear out the bags and nothing else that may be set
        // since the purpose of this driver is to share a handler
        foreach ($this->bags as $bag) {
            $bag->clear();
        }
        // reconnect the bags to the session
        $this->loadSession();
    }
}
